(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms')) :
    typeof define === 'function' && define.amd ? define('@ctrl/ngx-codemirror', ['exports', '@angular/core', '@angular/forms'], factory) :
    (global = global || self, factory((global.ctrl = global.ctrl || {}, global.ctrl['ngx-codemirror'] = {}), global.ng.core, global.ng.forms));
}(this, function (exports, core, forms) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }

    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(metadataKey, metadataValue);
    }

    function normalizeLineEndings(str) {
        if (!str) {
            return str;
        }
        return str.replace(/\r\n|\r/g, '\n');
    }
    /* tslint:enable */
    var CodemirrorComponent = /** @class */ (function () {
        function CodemirrorComponent(_differs, _ngZone) {
            this._differs = _differs;
            this._ngZone = _ngZone;
            /* class applied to the created textarea */
            this.className = '';
            /* name applied to the created textarea */
            this.name = 'codemirror';
            /* autofocus setting applied to the created textarea */
            this.autoFocus = false;
            /* preserve previous scroll position after updating value */
            this.preserveScrollPosition = false;
            /* called when the text cursor is moved */
            this.cursorActivity = new core.EventEmitter();
            /* called when the editor is focused or loses focus */
            this.focusChange = new core.EventEmitter();
            /* called when the editor is scrolled */
            this.scroll = new core.EventEmitter();
            this.value = '';
            this.disabled = false;
            this.isFocused = false;
            /** Implemented as part of ControlValueAccessor. */
            this.onChange = function (_) { };
            /** Implemented as part of ControlValueAccessor. */
            this.onTouched = function () { };
        }
        CodemirrorComponent_1 = CodemirrorComponent;
        Object.defineProperty(CodemirrorComponent.prototype, "options", {
            /**
             * set options for codemirror
             * @link http://codemirror.net/doc/manual.html#config
             */
            set: function (value) {
                this._options = value;
                if (!this._differ && value) {
                    this._differ = this._differs.find(value).create();
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CodemirrorComponent.prototype, "codeMirrorGlobal", {
            get: function () {
                if (this._codeMirror) {
                    return this._codeMirror;
                }
                this._codeMirror = require('codemirror');
                return this._codeMirror;
            },
            enumerable: true,
            configurable: true
        });
        CodemirrorComponent.prototype.ngAfterViewInit = function () {
            var _this = this;
            if (!this.ref) {
                return;
            }
            // in order to allow for universal rendering, we import Codemirror runtime with `require` to prevent node errors
            this._ngZone.runOutsideAngular(function () {
                _this.codeMirror = _this.codeMirrorGlobal.fromTextArea(_this.ref.nativeElement, _this._options);
                _this.codeMirror.on('cursorActivity', function (cm) {
                    return _this._ngZone.run(function () { return _this.cursorActive(cm); });
                });
                _this.codeMirror.on('scroll', _this.scrollChanged.bind(_this));
                _this.codeMirror.on('blur', function () {
                    return _this._ngZone.run(function () { return _this.focusChanged(false); });
                });
                _this.codeMirror.on('focus', function () {
                    return _this._ngZone.run(function () { return _this.focusChanged(true); });
                });
                _this.codeMirror.on('change', function (cm, change) {
                    return _this._ngZone.run(function () { return _this.codemirrorValueChanged(cm, change); });
                });
                _this.codeMirror.setValue(_this.value);
            });
        };
        CodemirrorComponent.prototype.ngDoCheck = function () {
            var _this = this;
            if (!this._differ) {
                return;
            }
            // check options have not changed
            var changes = this._differ.diff(this._options);
            if (changes) {
                changes.forEachChangedItem(function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                });
                changes.forEachAddedItem(function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                });
                changes.forEachRemovedItem(function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                });
            }
        };
        CodemirrorComponent.prototype.ngOnDestroy = function () {
            // is there a lighter-weight way to remove the cm instance?
            if (this.codeMirror) {
                this.codeMirror.toTextArea();
            }
        };
        CodemirrorComponent.prototype.codemirrorValueChanged = function (cm, change) {
            if (change.origin !== 'setValue') {
                this.value = cm.getValue();
                this.onChange(this.value);
            }
        };
        CodemirrorComponent.prototype.setOptionIfChanged = function (optionName, newValue) {
            if (!this.codeMirror) {
                return;
            }
            this.codeMirror.setOption(optionName, newValue);
        };
        CodemirrorComponent.prototype.focusChanged = function (focused) {
            this.onTouched();
            this.isFocused = focused;
            this.focusChange.emit(focused);
        };
        CodemirrorComponent.prototype.scrollChanged = function (cm) {
            this.scroll.emit(cm.getScrollInfo());
        };
        CodemirrorComponent.prototype.cursorActive = function (cm) {
            this.cursorActivity.emit(cm);
        };
        /** Implemented as part of ControlValueAccessor. */
        CodemirrorComponent.prototype.writeValue = function (value) {
            if (value === null || value === undefined) {
                return;
            }
            if (!this.codeMirror) {
                this.value = value;
                return;
            }
            var cur = this.codeMirror.getValue();
            if (value !== cur &&
                normalizeLineEndings(cur) !== normalizeLineEndings(value)) {
                this.value = value;
                if (this.preserveScrollPosition) {
                    var prevScrollPosition = this.codeMirror.getScrollInfo();
                    this.codeMirror.setValue(this.value);
                    this.codeMirror.scrollTo(prevScrollPosition.left, prevScrollPosition.top);
                }
                else {
                    this.codeMirror.setValue(this.value);
                }
            }
        };
        /** Implemented as part of ControlValueAccessor. */
        CodemirrorComponent.prototype.registerOnChange = function (fn) {
            this.onChange = fn;
        };
        /** Implemented as part of ControlValueAccessor. */
        CodemirrorComponent.prototype.registerOnTouched = function (fn) {
            this.onTouched = fn;
        };
        /** Implemented as part of ControlValueAccessor. */
        CodemirrorComponent.prototype.setDisabledState = function (isDisabled) {
            this.disabled = isDisabled;
            this.setOptionIfChanged('readOnly', this.disabled);
        };
        var CodemirrorComponent_1;
        __decorate([
            core.Input(),
            __metadata("design:type", Object)
        ], CodemirrorComponent.prototype, "className", void 0);
        __decorate([
            core.Input(),
            __metadata("design:type", Object)
        ], CodemirrorComponent.prototype, "name", void 0);
        __decorate([
            core.Input(),
            __metadata("design:type", Object)
        ], CodemirrorComponent.prototype, "autoFocus", void 0);
        __decorate([
            core.Input(),
            __metadata("design:type", Object),
            __metadata("design:paramtypes", [Object])
        ], CodemirrorComponent.prototype, "options", null);
        __decorate([
            core.Input(),
            __metadata("design:type", Object)
        ], CodemirrorComponent.prototype, "preserveScrollPosition", void 0);
        __decorate([
            core.Output(),
            __metadata("design:type", Object)
        ], CodemirrorComponent.prototype, "cursorActivity", void 0);
        __decorate([
            core.Output(),
            __metadata("design:type", Object)
        ], CodemirrorComponent.prototype, "focusChange", void 0);
        __decorate([
            core.Output(),
            __metadata("design:type", Object)
        ], CodemirrorComponent.prototype, "scroll", void 0);
        __decorate([
            core.ViewChild('ref', { static: true }),
            __metadata("design:type", core.ElementRef)
        ], CodemirrorComponent.prototype, "ref", void 0);
        CodemirrorComponent = CodemirrorComponent_1 = __decorate([
            core.Component({
                selector: 'ngx-codemirror',
                template: "\n  <textarea\n    [name]=\"name\"\n    class=\"ngx-codemirror {{ className }}\"\n    [class.ngx-codemirror--focused]=\"isFocused\"\n    autocomplete=\"off\"\n    [autofocus]=\"autoFocus\"\n    #ref>\n  </textarea>\n  ",
                providers: [
                    {
                        provide: forms.NG_VALUE_ACCESSOR,
                        useExisting: core.forwardRef(function () { return CodemirrorComponent_1; }),
                        multi: true,
                    },
                ],
                preserveWhitespaces: false,
                changeDetection: core.ChangeDetectionStrategy.OnPush
            }),
            __metadata("design:paramtypes", [core.KeyValueDiffers, core.NgZone])
        ], CodemirrorComponent);
        return CodemirrorComponent;
    }());

    var CodemirrorModule = /** @class */ (function () {
        function CodemirrorModule() {
        }
        CodemirrorModule = __decorate([
            core.NgModule({
                exports: [CodemirrorComponent],
                declarations: [CodemirrorComponent],
            })
        ], CodemirrorModule);
        return CodemirrorModule;
    }());

    exports.CodemirrorComponent = CodemirrorComponent;
    exports.CodemirrorModule = CodemirrorModule;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=ctrl-ngx-codemirror.umd.js.map
